--
-- PoettingerAlpha
-- Specialization for PoettingerAlpha
--
-- @author  	fruktor 
-- @version 	v0.2
-- @date  		05/05/13
-- @history:	v0.1 - Initial version
--				v0.2 - convert to fs13
--

local mod_directory = g_currentModDirectory;
source(Utils.getFilename("StaticParticleSystem.lua", mod_directory));

source(Utils.getFilename("SetWorkingTransportStateEvent.lua", mod_directory));
source(Utils.getFilename("SetLiftUpDownStateEvent.lua", mod_directory));

 
PoettingerAlpha = {};

function PoettingerAlpha.prerequisitesPresent(specializations)
    Vehicle.registerJointType("FastCoupling");
	return true;
end;

function PoettingerAlpha:load(xmlFile)

	self.setLiftUp = SpecializationUtil.callSpecializationsFunction("setLiftUp");
	self.setTurnedOn = SpecializationUtil.callSpecializationsFunction("setTurnedOn");
	--self.setSwath = SpecializationUtil.callSpecializationsFunction("setSwath");
	--self.setCollector = SpecializationUtil.callSpecializationsFunction("setCollector");
	self.setTransport = SpecializationUtil.callSpecializationsFunction("setTransport");
	--self.setSelection = SpecializationUtil.callSpecializationsFunction("setSelection");
	self.setPSstate = SpecializationUtil.callSpecializationsFunction("setPSstate");
	
	self.setWorkingTransportState = SpecializationUtil.callSpecializationsFunction("setWorkingTransportState");
	self.setLiftUpDownState = SpecializationUtil.callSpecializationsFunction("setLiftUpDownState");
	

	self.attacherJoint.jointType = Vehicle.JOINTTYPE_FASTCOUPLING;
	
	self.alpMot = {};
	
	self.alpMot.left = {};
	self.alpMot.left.node = Utils.indexToObject( self.components, getXMLString(xmlFile, "vehicle.PoetAlpha.left#idx") );
	self.alpMot.left.rotMin = Utils.getVectorNFromString(getXMLString(xmlFile, "vehicle.PoetAlpha.left#rotMin")); 
	self.alpMot.left.rotMax = Utils.getVectorNFromString(getXMLString(xmlFile, "vehicle.PoetAlpha.left#rotMax")); 
	for i=1,3 do
		self.alpMot.left.rotMin[i] = Utils.degToRad(self.alpMot.left.rotMin[i]);
		self.alpMot.left.rotMax[i] = Utils.degToRad(self.alpMot.left.rotMax[i]);
	end;
	self.alpMot.right = {};
	self.alpMot.right.node = Utils.indexToObject( self.components, getXMLString(xmlFile, "vehicle.PoetAlpha.right#idx") );
	self.alpMot.right.rotMin = Utils.getVectorNFromString(getXMLString(xmlFile, "vehicle.PoetAlpha.right#rotMin")); 
	self.alpMot.right.rotMax = Utils.getVectorNFromString(getXMLString(xmlFile, "vehicle.PoetAlpha.right#rotMax")); 
	for i=1,3 do
		self.alpMot.right.rotMin[i] = Utils.degToRad(self.alpMot.right.rotMin[i]);
		self.alpMot.right.rotMax[i] = Utils.degToRad(self.alpMot.right.rotMax[i]);
	end;	
	self.alpMot.walze = Utils.indexToObject( self.components, getXMLString(xmlFile, "vehicle.PoetAlpha.walze#idx") );
	self.alpMot.halter = {};
	self.alpMot.halter.node = Utils.indexToObject( self.components, getXMLString(xmlFile, "vehicle.PoetAlpha.halter#idx") );
	self.alpMot.halter.rotMin = Utils.getVectorNFromString(getXMLString(xmlFile, "vehicle.PoetAlpha.halter#rotMin")); 
	self.alpMot.halter.rotMax = Utils.getVectorNFromString(getXMLString(xmlFile, "vehicle.PoetAlpha.halter#rotMax")); 
	for i=1,3 do
		self.alpMot.halter.rotMin[i] = Utils.degToRad(self.alpMot.halter.rotMin[i]);
		self.alpMot.halter.rotMax[i] = Utils.degToRad(self.alpMot.halter.rotMax[i]);
	end;	

	self.alpMot.groundReferenceNode = Utils.indexToObject( self.components, getXMLString(xmlFile, "vehicle.PoetAlpha.groundReference#idx"));
	self.alpMot.groundReferenceThreshold = getXMLFloat(xmlFile, "vehicle.PoetAlpha.groundReference#thresh");
	
	self.alpMot.cuttingAreas = {};
	self.alpMot.cuttingAreasSend = {};
	local area = {};
	area.start = Utils.indexToObject( self.components, getXMLString(xmlFile, "vehicle.PoetAlpha.cutArea#startIndex"));
	area.width = Utils.indexToObject( self.components, getXMLString(xmlFile, "vehicle.PoetAlpha.cutArea#widthIndex"));
	area.height = Utils.indexToObject( self.components, getXMLString(xmlFile, "vehicle.PoetAlpha.cutArea#heightIndex"));
	table.insert(self.alpMot.cuttingAreas, area);
	table.insert(self.alpMot.cuttingAreasSend, area);
	self.alpMot.windrowDropAreas = {};
	local area = {};
	area.start = Utils.indexToObject( self.components, getXMLString(xmlFile, "vehicle.PoetAlpha.windrowDropArea#startIndex"));
	area.width = Utils.indexToObject( self.components, getXMLString(xmlFile, "vehicle.PoetAlpha.windrowDropArea#widthIndex"));
	area.height = Utils.indexToObject( self.components, getXMLString(xmlFile, "vehicle.PoetAlpha.windrowDropArea#heightIndex"));			
	area.value = 0;
	area.valueAccum = 0;
	table.insert(self.alpMot.windrowDropAreas, area);	
	
	-- ps
	self.alpMot.cutPS = {};
	self.alpMot.cutPS.ps = {};
	self.alpMot.cutPS.active = false;
	baseName = "vehicle.PoetAlpha.PScut";
	Utils.loadParticleSystem(xmlFile, self.alpMot.cutPS.ps, baseName, self.components, false, nil, self.baseDirectory);			
	--StaticParticleSystem.loadParticleSystem(xmlFile, self.alpMot.cutPS.ps, baseName, self.components, false, nil, self.baseDirectory);			
	self.alpMot.cutPS.node = Utils.indexToObject( self.components, getXMLString(xmlFile, "vehicle.PoetAlpha.PScut#node"));	
	self.psTime = 300;
	self.psTimeT = 300;

	self.alpMot.setTransport = true;
	self.alpMot.isWorking = false;
	self.alpMot.isTransport = true;
	
	self.alpMot.setTurnedOn = false;
	self.alpMot.setSwath = true;
	
	self.alpMot.setLiftUp = true;
	self.alpMot.isLiftUp = true;
	self.alpMot.isLiftDown = false;
	self.alpMot.currentTransLimit = {0};
	self.alpMot.maxTrans = {0.5};
	self.alpMot.minTrans = {0};
	
	self.accumulatedCuttingAreaValues = {0};
	
	self.alpMot.curFCjointRotLimit = {};
	self.alpMot.curFCjointRotLimit.z = 0;
	
	-- "dynamic" plane, aka the flagshader from SRS
	--<dynPlane idx="1>0|5">
	self.alpMot.dynPlanes = {};
	local i=0;
	while true do
		i = i + 1;
		local node = Utils.indexToObject( self.components, getXMLString(xmlFile, string.format("vehicle.PoetAlpha.dynPlane%d#idx", i)));
		if node == nil then break; end;
		local scale = Utils.getNoNil( getXMLBool(xmlFile,  string.format("vehicle.PoetAlpha.dynPlane%d#scale", i)), false );
		local e = {};
		e.node = node;
		e.scale = scale;
		table.insert(self.alpMot.dynPlanes, e);
	end;
	self.alpMot.dynPlanesActive = true;

	
	-- sound
	local mowerStartSound = getXMLString(xmlFile, string.format("vehicle.mowerStartSound#file"));
	mowerStartSound = Utils.getFilename(mowerStartSound, self.baseDirectory); 
	self.mowerStartSound = {};
	self.mowerStartSound = createSample(string.format("mowerStartSound"));
	loadSample(self.mowerStartSound, mowerStartSound, false);
	self.mowerStartSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, string.format("vehicle.mowerStartSound#pitchOffset")), 1);
	self.mowerStartSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, string.format("vehicle.mowerStartSound#volume")), 1);
	self.mowerStartSoundActive = false;	
	self.mowerStartDuration = getSampleDuration(self.mowerStartSound);
	
	-- sound
	local mowerLoopSound = getXMLString(xmlFile, string.format("vehicle.mowerLoopSound#file"));
	mowerLoopSound = Utils.getFilename(mowerLoopSound, self.baseDirectory); 
	self.mowerLoopSound = {};
	self.mowerLoopSound = createSample(string.format("mowerLoopSound"));
	loadSample(self.mowerLoopSound, mowerLoopSound, false);
	self.mowerLoopSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, string.format("vehicle.mowerLoopSound#pitchOffset")), 1);
	self.mowerLoopSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, string.format("vehicle.mowerLoopSound#volume")), 1);
	self.mowerLoopSoundActive = false;
	self.mowerLoopTime = -1;	

	-- sound
	local mowerStopSound = getXMLString(xmlFile, string.format("vehicle.mowerStopSound#file"));
	mowerStopSound = Utils.getFilename(mowerStopSound, self.baseDirectory); 
	self.mowerStopSound = {};
	self.mowerStopSound = createSample(string.format("mowerStopSound"));
	loadSample(self.mowerStopSound, mowerStopSound, false);
	self.mowerStopSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, string.format("vehicle.mowerStopSound#pitchOffset")), 1);
	self.mowerStopSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, string.format("vehicle.mowerStopSound#volume")), 1);
	self.mowerStopSoundActive = false;		
	self.mowerStopDuration = getSampleDuration(self.mowerStopSound);
	self.mowerLoopEndTime = -1;	-- used to syn the rolls' movement
	
	--# MR 
	self.realMowerWorkingPowerConsumption = 20;	
	self.realCurrentPowerConsumption = 0;
	self.lastMowerArea = 0;
	self.realMowerWorkingPowerConsumptionInc = 90;
	self.realMowerTmpAreaS = 0;

	self.realMowerUseResistance = true;
	self.realMowerWorkingResistance = 3;
	self.realCurrentMowerWorkingResistance = 0;
	
end

function PoettingerAlpha:delete()
	stopSample(self.mowerStartSound);
	stopSample(self.mowerLoopSound);
	stopSample(self.mowerStopSound);
	delete(self.mowerStartSound);
	delete(self.mowerLoopSound);
	delete(self.mowerStopSound);
	Utils.deleteParticleSystem(self.alpMot.cutPS.ps);
end;

function PoettingerAlpha:mouseEvent(posX, posY, isDown, isUp, button)
end;

function PoettingerAlpha:keyEvent(unicode, sym, modifier, isDown)
end;

function PoettingerAlpha:readStream(streamId, connection)
	self.alpMot.setTransport = streamReadBool(streamId);
	self.alpMot.isWorking = streamReadBool(streamId);
	self.alpMot.isTransport = streamReadBool(streamId);
	self.alpMot.setTurnedOn = streamReadBool(streamId);
	self.alpMot.setLiftUp = streamReadBool(streamId);
	self.alpMot.isLiftUp = streamReadBool(streamId);
	self.alpMot.isLiftDown = streamReadBool(streamId);
	local rx = streamReadFloat32(streamId);
	local ry = streamReadFloat32(streamId);
	local rz = streamReadFloat32(streamId);
	
	local newVal = {0, 0, 0};
	--[[if self.alpMot.setTransport == true then
		newVal = self.alpMot.left.rotMax;
	else
		newVal = self.alpMot.left.rotMin;
	end;]]--
	setRotation(self.alpMot.left.node, rx, ry, rz);
	setRotation(self.alpMot.right.node, rx, ry, -rz);
	
	for i,plane in pairs(self.alpMot.dynPlanes) do
		if plane.scale then
			if self.alpMot.setTransport then
				setScale(plane.node, 1, 0.5, 1);
			else
				setScale(plane.node, 1, 1.0, 1);
			end;
		end;
	end;	
end;


function PoettingerAlpha:writeStream(streamId, connection)
	streamWriteBool(streamId, self.alpMot.setTransport);
	streamWriteBool(streamId, self.alpMot.isWorking);
	streamWriteBool(streamId, self.alpMot.isTransport);
	streamWriteBool(streamId, self.alpMot.setTurnedOn);
	streamWriteBool(streamId, self.alpMot.setLiftUp);
	streamWriteBool(streamId, self.alpMot.isLiftUp);
	streamWriteBool(streamId, self.alpMot.isLiftDown);
	local rx, ry, rz = getRotation(self.alpMot.left.node);
	streamWriteFloat32(streamId, rx);
	streamWriteFloat32(streamId, ry);
	streamWriteFloat32(streamId, rz);
	
end;

function PoettingerAlpha:readStreamUpdate(streamId, connection)

	
end;

function PoettingerAlpha:writeStreamUpdate(streamId, connection)
end;


function PoettingerAlpha:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)

	self.alpMot.setTurnedOn = Utils.getNoNil( getXMLBool(xmlFile,key..string.format("#alpMot.setTurnedOn")), false);
	self.alpMot.setTransport = Utils.getNoNil( getXMLBool(xmlFile,key..string.format("#alpMot.setTransport")), false);
	self.alpMot.isWorking = Utils.getNoNil( getXMLBool(xmlFile,key..string.format("#alpMot.isWorking")), false);
	self.alpMot.isTransport = Utils.getNoNil( getXMLBool(xmlFile,key..string.format("#alpMot.isTransport")), false);
	self.alpMot.setLiftUp = Utils.getNoNil( getXMLBool(xmlFile,key..string.format("#alpMot.setLiftUp")), false);
	self.alpMot.isLiftUp = Utils.getNoNil( getXMLBool(xmlFile,key..string.format("#alpMot.isLiftUp")), false);
	self.alpMot.isLiftDown = Utils.getNoNil( getXMLBool(xmlFile,key..string.format("#alpMot.isLiftDown")), false);
	
	local newY = Utils.getNoNil( getXMLFloat(xmlFile,key..string.format("#alpMot.comp1.jointNodeY")), 0.50691); ---0.28515);
	local tx,ty,tz = getTranslation( self.componentJoints[1].jointNode );
	--print("tx/ty/tz="..tx.." "..ty.." "..tz);
	setTranslation(self.componentJoints[1].jointNode, tx, newY, tz);
	setJointFrame(self.componentJoints[1].jointIndex, 0, self.componentJoints[1].jointNode);		
	
	self.alpMot.currentTransLimit[1] = Utils.getNoNil( getXMLFloat(xmlFile,key..string.format("#alpMot.comp1.transLimit")), 0);
	setJointTranslationLimit(self.componentJoints[1].jointIndex, 1, true, -self.alpMot.currentTransLimit[1], 0);
	local fac = ((self.alpMot.currentTransLimit[1]-self.alpMot.minTrans[1])/(self.alpMot.maxTrans[1]-self.alpMot.minTrans[1]));
	setJointRotationLimit(self.componentJoints[1].jointIndex, 0, true, 0, fac*Utils.degToRad(16));
	setJointRotationLimit(self.componentJoints[1].jointIndex, 2, true, 0, fac*Utils.degToRad(16));	
	
	self.alpMot.curFCjointRotLimit.z = Utils.getNoNil( getXMLFloat(xmlFile,key..string.format("#alpMot.fc.rotLimit")), 0);
	if self.fastCouplerJoint ~= nil and self.fastCouplerJoint.jointIndex ~= nil then
		setJointRotationLimit(self.fastCouplerJoint.jointIndex, 2, true, -self.alpMot.curFCjointRotLimit.z, self.alpMot.curFCjointRotLimit.z);
	end;
	
	local newVal = {0, 0, 0};
	if self.alpMot.setTransport == true then
		newVal = self.alpMot.left.rotMax;
	else
		newVal = self.alpMot.left.rotMin;
	end;
	setRotation(self.alpMot.left.node, newVal[1], newVal[2], newVal[3]);
	setRotation(self.alpMot.right.node, newVal[1], newVal[2], -newVal[3]);	
	
	for i,plane in pairs(self.alpMot.dynPlanes) do
		if plane.scale then
			if self.alpMot.setTransport then
				setScale(plane.node, 1, 0.5, 1);
			else
				setScale(plane.node, 1, 1.0, 1);
			end;
		end;
	end;	
	return BaseMission.VEHICLE_LOAD_OK;
end;

function PoettingerAlpha:getSaveAttributesAndNodes(nodeIdent)

	local attributes = ' ';
	local mystring = 	
		' alpMot.setTurnedOn="'..tostring(self.alpMot.setTurnedOn)..'"'..
		' alpMot.setTransport="'..tostring(self.alpMot.setTransport)..'"'..
		' alpMot.isWorking="'..tostring(self.alpMot.isWorking)..'"'..
		' alpMot.isTransport="'..tostring(self.alpMot.isTransport)..'"'..
		' alpMot.setLiftUp="'..tostring(self.alpMot.setLiftUp)..'"'..
		' alpMot.isLiftUp="'..tostring(self.alpMot.isLiftUp)..'"'..
		' alpMot.isLiftDown="'..tostring(self.alpMot.isLiftDown)..'"';
	attributes = attributes .. mystring;
	
	local x,y,z = getTranslation(self.componentJoints[1].jointNode);
	attributes = attributes .. string.format(' alpMot.comp1.jointNodeY="%f"', y);	
	attributes = attributes .. string.format(' alpMot.comp1.transLimit="%f"', self.alpMot.currentTransLimit[1]);
	
	attributes = attributes .. string.format(' alpMot.fc.rotLimit="%f"', self.alpMot.curFCjointRotLimit.z);
	
	local node = nil;
    return attributes, node;
end;


function PoettingerAlpha:update(dt)

	--for k, part in pairs(self.movingParts) do
	--	part.isDirty = true;
	--end;
	


	if self:getIsActiveForInput() then
		if InputBinding.hasEvent(InputBinding.LOWER_IMPLEMENT) then
			if self.alpMot.isWorking then
				self:setLiftUp( not self.alpMot.setLiftUp );
			end;
		end;
		if InputBinding.hasEvent(InputBinding.POETX_TURNONOFF) then
			if self.alpMot.isWorking then
				self:setTurnedOn( not self.alpMot.setTurnedOn );
			end;
		end;
		if InputBinding.hasEvent(InputBinding.POETX_TRANSPORT) then
			if self.alpMot.isLiftUp then
				self:setTransport( not self.alpMot.setTransport );
			end;
		end;

	end;

	if self.alpMot.isWorking == false and self.alpMot.setTurnedOn == true then
		self:setTurnedOn( false );
	end;	
	
	if self.attacherVehicle ~= nil then
		if self.attacherVehicle.attacherVehicle ~= nil then
			if self.alpMot.setTurnedOn then
				if not self.attacherVehicle.attacherVehicle.isMotorStarted then
					self:setTurnedOn(false);
				end;
			end;
		end;
	end;	
		
	self.wasToFast = false;

	if self:getIsActive() then
			
		local toFast = self:doCheckSpeedLimit() and self.lastSpeed*3600 > 31;		
		local cuttingAreasSend = {};
		local winCutAreasSend = {};
		local winDropAreasSend = {};
		local validCutter = false;

		-- transport
		if self.alpMot.isLiftUp then
			
			if (self.alpMot.setTransport == true and self.alpMot.isTransport == false) or 
				(self.alpMot.setTransport == false and self.alpMot.isWorking == false) then				
				
				for i,plane in pairs(self.alpMot.dynPlanes) do
					if plane.scale then
						local sx, sy, sz = getScale(plane.node)
						local newS = Utils.getMovedLimitedValues({sy}, {1.0}, {0.5}, 1, 4000, dt, self.alpMot.setTransport);
						setScale(plane.node, 1, newS[1], 1);
					end;
				end;				
					
				local x,y,z = getTranslation(self.componentJoints[1].jointNode);
				--local newY = Utils.getMovedLimitedValues({y}, {-0.28515}, {-0.12023}, 1, 2000, dt, self.alpMot.setTransport);
				local newY = Utils.getMovedLimitedValues({y}, {0.34199}, {0.50691}, 1, 2000, dt, not self.alpMot.setTransport);
				
				if self.isServer then
					setTranslation(self.componentJoints[1].jointNode, x, newY[1], z);
					setJointFrame(self.componentJoints[1].jointIndex, 0, self.componentJoints[1].jointNode);				
				end;
				
				local rx, ry, rz = getRotation(self.alpMot.left.node);
				local newVal = Utils.getMovedLimitedValues({rx, ry, rz}, self.alpMot.left.rotMin, self.alpMot.left.rotMax, 3, 4000, dt, self.alpMot.setTransport);
				setRotation(self.alpMot.left.node, newVal[1], newVal[2], newVal[3]);
				setRotation(self.alpMot.right.node, newVal[1], newVal[2], -newVal[3]);
				
				if self.isServer then 
					local isWorking = false;
					local isTransport = false;
					local chk = 0;
					for i=1,3 do
						if (newVal[i] == self.alpMot.left.rotMin[i])then 
							chk = chk + 1;
						end;
					end;
					if chk == 3 then
						isWorking = true;
					end;
					if not isWorking then
						chk = 0;
						for i=1,3 do
							if (newVal[i] == self.alpMot.left.rotMax[i]) then 
								chk = chk + 1;
							end;
						end;
						if chk == 3 then
							isTransport = true;
						end;
					end;
					--print(tostring(self).."rz."..tostring(rz)..": "..tostring(self.alpMot.left.rotMin[3]).." "..tostring(self.alpMot.left.rotMax[3]).." "..tostring(self.alpMot.isWorking).." "..tostring(self.alpMot.isTransport));
					if self.alpMot.isWorking ~= isWorking or self.alpMot.isTransport ~= isTransport then
						self:setWorkingTransportState( isWorking, isTransport );
					end;
				end;
				
			end;
				
		end;
			
		-- lift
		if self.alpMot.isWorking then 

			if (self.alpMot.setLiftUp == true and self.alpMot.isLiftUp == false) or 
				(self.alpMot.setLiftUp == false and self.alpMot.isLiftDown == false) then
				
				local rz = Utils.getMovedLimitedValues({self.alpMot.curFCjointRotLimit.z}, {Utils.degToRad(16)}, {0}, 1, 1500, dt, self.alpMot.setLiftUp);	
				setJointRotationLimit(self.fastCouplerJoint.jointIndex, 2, true, -rz[1], rz[1]);
				self.alpMot.curFCjointRotLimit.z = rz[1];					
				
				local newTransLimit = Utils.getMovedLimitedValues(self.alpMot.currentTransLimit, self.alpMot.maxTrans, self.alpMot.minTrans, 1, 1500, dt, self.alpMot.setLiftUp);
				--if math.abs(newTransLimit[1] * 100 - self.alpMot.currentTransLimit[1]*100) > 0.001 then		
				if self.isServer then				
					setJointTranslationLimit(self.componentJoints[1].jointIndex, 1, true, -newTransLimit[1], 0);
					local fac = ((newTransLimit[1]-self.alpMot.minTrans[1])/(self.alpMot.maxTrans[1]-self.alpMot.minTrans[1]));
					setJointRotationLimit(self.componentJoints[1].jointIndex, 0, true, 0, fac*Utils.degToRad(16));
					setJointRotationLimit(self.componentJoints[1].jointIndex, 2, true, 0, fac*Utils.degToRad(16));					
				end;
				--end;		
				self.alpMot.currentTransLimit = newTransLimit;		
				
				local x,y,z = getTranslation(self.componentJoints[1].jointNode);
				--local newY = Utils.getMovedLimitedValues({y}, {-0.28515}, {-0.12023}, 1, 2000, dt, self.alpMot.setTransport);
				local newY = Utils.getMovedLimitedValues({y}, {0.34199}, {0.50691}, 1, 2000, dt, not self.alpMot.setTransport);				
				if self.isServer then
					setTranslation(self.componentJoints[1].jointNode, x, newY[1], z);
					setJointFrame(self.componentJoints[1].jointIndex, 0, self.componentJoints[1].jointNode);				
				end;
				
				if self.isServer then
					local isLiftDown = false;
					local isLiftUp = false;
					local chk = 0;
					for i=1,1 do
						if (self.alpMot.currentTransLimit[i] == self.alpMot.maxTrans[i]) then 
							chk = chk + 1;
						end;
					end;
					if chk == 1 then
						isLiftDown = true;
					end;
					if not isLiftDown then
						chk = 0;
						for i=1,1 do
							if (self.alpMot.currentTransLimit[i] == self.alpMot.minTrans[i]) then 
								chk = chk + 1;
							end;
						end;
						if chk == 1 then
							isLiftUp = true;
						end;
					end;
					--print(tostring(self).."lift."..tostring(self.alpMot.isLiftUp).." "..tostring(self.alpMot.isLiftDown));
					if self.alpMot.isLiftDown ~= isLiftDown or self.alpMot.isLiftUp ~= isLiftUp then
						self:setLiftUpDownState( isLiftUp, isLiftDown );
					end;
				end;
				
			end;				
			
		end;
			
			
		if self.alpMot.setTurnedOn then
			if self.mowerStartSoundActive then
				local fac = 1.0;
				if self.mowerLoopTime > self.time then
					fac = 1.0 - (self.mowerLoopTime - self.time) / self.mowerStartDuration;
				end;
				rotate( self.alpMot.walze, fac*Utils.degToRad(10), 0, 0 );
			else
				rotate( self.alpMot.walze, 1*Utils.degToRad(10), 0, 0 );
			end;
		else
			if self.mowerLoopTime == -1 then
				if self.mowerLoopEndTime > self.time then
					local fac = (self.mowerLoopEndTime - self.time) / self.mowerStopDuration;
					rotate( self.alpMot.walze, fac*Utils.degToRad(10), 0, 0 );
				end;
			end;			
		end;
		
	end;	
	
	
	--# MR
	if self.isServer and self.isActive then
		if self.realMowerUseResistance and self.realCurrentMowerWorkingResistance>0 and self.realGroundSpeed>0.1 then
			local force = -self.realCurrentMowerWorkingResistance*dt/1000/ self.realGroundSpeed;-- direction vector = velocity vector					
			addForce(self.realMainComponent.node, force*self.realVelX, force*self.realVelY, force*self.realVelZ, self.realComX, self.realComY, self.realComZ, true);	
		end;
	end;	
	
	
end;


function PoettingerAlpha:updateTick(dt)

	
	
	--if self.fastCouplerJoint ~= nil and self.isExpanded then
	_=[[if self.attacherVehicle ~= nil then
		if self.fastCouplerJoint ~= nil then
			self.fastCouplerJoint.moveDown = false;
			self.attacherVehicle.needsLowering = false;
		else
			self.attacherVehicle.needsLowering = true;
		end;	
	end; ]]
	
	--if self.mowerIsAttached and self.attacherVehicle ~= nil then
	if self.fastCouplerJoint ~= nil and self.attacherVehicle ~= nil then
	
		local spd = self.lastSpeed*3600;		
		
		if spd < 5 and self.alpMot.dynPlanesActive == true then
			self.alpMot.dynPlanesActive = false;
			for i,plane in pairs(self.alpMot.dynPlanes) do
				setShaderParameter(plane.node, "displacementSpeedFrequencyAndTexOff", 0.1, 0, 0, 0.05,false);
			end;
		elseif self.alpMot.dynPlanesActive == false and spd > 5 then
			self.alpMot.dynPlanesActive = true;
			for i,plane in pairs(self.alpMot.dynPlanes) do
				--0.1 4 0.9 0.05			
				--print(".");
				setShaderParameter(plane.node, "displacementSpeedFrequencyAndTexOff", 0.1, 4, 0.9, 0.05,false);
			end;
		end;
		
		if self.fastCouplerJoint ~= nil then
			self.fastCouplerJoint.moveDown = false;
		end;		
	else
		local spd = self.lastSpeed*3600;
		if spd < 5 and self.alpMot.dynPlanesActive == true then
			self.alpMot.dynPlanesActive = false;
			for i,plane in pairs(self.alpMot.dynPlanes) do
				setShaderParameter(plane.node, "displacementSpeedFrequencyAndTexOff", 0.1, 0, 0, 0.05,false);
			end;
		end;
	
		--self.fastCouplerJoint.moveDown = false;
		
		--self.attacherJoint.allowsJointTransLimitMovement = false;
		--self.attacherJoint.allowsJointRotLimitMovement = true;
		
		--self.attacherVehicle.attacherJoint.allowsJointTransLimitMovement = false;
		--self.attacherVehicle.attacherJoint.allowsJointRotLimitMovement = true;
		
		if self.fastCouplerJoint ~= nil then
			self.fastCouplerJoint.moveDown = false;
		end;
		

	end;	


	if self:getIsActive() then		
		self.realCurrentMowerWorkingResistance = 0;
		local validCutter = false;
		local toFast = self:doCheckSpeedLimit() and self.lastSpeed*3600 > 31;
		if not toFast then
			if self.isServer and self.alpMot.setTurnedOn then
				local x,y,z = getWorldTranslation(self.alpMot.groundReferenceNode);
				local terrainHeight = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x, 0, z);
				if terrainHeight+self.alpMot.groundReferenceThreshold >= y then
					local foldAnimTime = self.foldAnimTime;

					local numAreasSend = 0;
					local numDropAreas = 0;
					local areasSend = self.alpMot.cuttingAreasSend;
					for _,area in pairs(self.alpMot.cuttingAreas) do
						if self:getIsAreaActive(area) then
							numAreasSend = numAreasSend + 1;
							areasSend[numAreasSend].x,_,areasSend[numAreasSend].z = getWorldTranslation(area.start);
							areasSend[numAreasSend].x1,_,areasSend[numAreasSend].z1 = getWorldTranslation(area.width);
							areasSend[numAreasSend].x2,_,areasSend[numAreasSend].z2 = getWorldTranslation(area.height);
							areasSend[numAreasSend].dropWindrow = area.dropWindrow;
							areasSend[numAreasSend].dropArea = area.dropArea;
							areasSend[numAreasSend].dropArea = 1;
							
							--local x,y,z = getWorldTranslation(area.start);
							--local x1,y1,z1 = getWorldTranslation(area.width);
							--local x2,y2,z2 = getWorldTranslation(area.height);														
							--local area1 = Utils.getFruitArea(FruitUtil.FRUITTYPE_GRASS, x, z, x1, z1, x2, z2);
							--local area2 = Utils.getFruitCutLongArea(FruitUtil.FRUITTYPE_GRASS, x, z, x1, z1, x2, z2);
							--local area3 = Utils.getFruitWindrowArea(FruitUtil.FRUITTYPE_GRASS, x, z, x1, z1, x2, z2);							
							--if area1 > 0 and self.lastSpeed*3600 > 2 then --or area2 > 0 or area3 > 0 then
							--	validCutter = true;
							--end;		

							self.realCurrentMowerWorkingResistance = self.realCurrentMowerWorkingResistance + self.realMowerWorkingResistance;							
						end
					end
					for _,area in pairs(self.alpMot.windrowDropAreas) do
						numDropAreas = numDropAreas + 1;
						area.x, _,area.z = getWorldTranslation(area.start);
						area.x1,_,area.z1 = getWorldTranslation(area.width);
						area.x2,_,area.z2 = getWorldTranslation(area.height);
					end

					if numAreasSend > 0 then
						local numAreasUsed, numDropAreasUsed, pickedUpWindrow = MowerAreaEvent.runLocally(areasSend, numAreasSend, self.alpMot.windrowDropAreas, numDropAreas);
						if numAreasUsed > 0 or numDropAreasUsed > 0 then
							validCutter = true;
							g_server:broadcastEvent(MowerAreaEvent:new(areasSend, numAreasUsed, self.alpMot.windrowDropAreas, numDropAreas, numDropAreasUsed));
						end
						--if pickedUpWindrow > 0 and self.allowFillType ~= nil and self:allowFillType(Fillable.FILLTYPE_GRASS_WINDROW) then
						--	local fillDelta = pickedUpWindrow * FruitUtil.getFillTypeLiterPerSqm(Fillable.FILLTYPE_GRASS_WINDROW, 1) * self.mowerPickupFillScale;
						--	self:setFillLevel(self:getFillLevel(Fillable.FILLTYPE_GRASS_WINDROW)+fillDelta, Fillable.FILLTYPE_GRASS_WINDROW);
						--end
					end;
				end;
			end;
		end;
		
		--# MR 
		self.realCurrentPowerConsumption = 0;
		if self.alpMot.setTurnedOn then -- the mower is active
			self.realCurrentPowerConsumption = self.realMowerWorkingPowerConsumption;
			
			self.realMowerTmpAreaS = 0.9*self.realMowerTmpAreaS + 0.1*self.lastMowerArea*g_currentMission:getFruitPixelsToSqm();			
			self.realCurrentPowerConsumption = self.realCurrentPowerConsumption + self.realMowerWorkingPowerConsumptionInc*self.realMowerTmpAreaS/(0.06*dt);-- now we do not use the speed to "dampen" the powerInc since 2013 version of the moreRealistic engine is more steady					
			
			--print(self.time .. " RealisticMower:updateTick - last area cut = " .. self.realTmpAreaS);
		else
			self.realMowerTmpAreaS = 0;
		end;	

		--#
		if self.isServer then
			local changed = false;
			local state = false;
			if validCutter == true then 
				self.psTime = 0;
			else
				self.psTime = self.psTime + dt;
			end;
			if self.psTime < self.psTimeT then
				state = true;
			else
				state = false;
			end;
			if state ~= self.psState then
				--self.oldPSstate = self.psState;
				changed = true;
			end;
			if changed then
				self:setPSstate(state);
			end;
		end;		
			
	end;
	
	if self.isClient then
		if self.oldPSstate ~= self.psState then
			if self.alpMot.setTurnedOn and self.psState then
				Utils.setEmittingState(self.alpMot.cutPS.ps, self.alpMot.setTurnedOn);
			else
				Utils.setEmittingState(self.alpMot.cutPS.ps, false);			
			end;
			self.oldPSstate = self.psState;
		end;	
	end;
	
	if self.isClient then		
		if self.alpMot.setTurnedOn and self:getIsActiveForSound() then
			if self.mowerStartSoundActive == false then
				setSamplePitch(self.mowerStartSound, self.mowerStartSoundPitchOffset);
				playSample(self.mowerStartSound, 1, self.mowerStartSoundVolume, 0);
				self.mowerStartSoundActive = true;
				self.mowerLoopTime = self.time + self.mowerStartDuration;
			end;
		else
			if self.mowerLoopSoundActive == true then
				stopSample(self.mowerLoopSound);
				self.mowerLoopSoundActive = false;
				self.mowerLoopTime = -1;
				self.mowerStartSoundActive = false;
				setSamplePitch(self.mowerStopSound, self.mowerStopSoundPitchOffset);
				playSample(self.mowerStopSound, 1, self.mowerStopSoundVolume, 0);		
				self.mowerLoopEndTime = self.time + self.mowerStopDuration;				
			end;		
		end;
		if self:getIsActiveForSound() then
			if self.mowerLoopTime ~= -1 then
				if self.time > self.mowerLoopTime and self.mowerLoopSoundActive == false then
					setSamplePitch(self.mowerLoopSound, self.mowerLoopSoundPitchOffset);
					playSample(self.mowerLoopSound, 0, self.mowerLoopSoundVolume, 0);
					self.mowerLoopSoundActive = true;				
				end;
			end;
		else 
			if self.mowerLoopSoundActive == true then
				stopSample(self.mowerLoopSound);
				self.mowerLoopSoundActive = false;
				self.mowerLoopTime = -1;
				self.mowerStartSoundActive = false;
				setSamplePitch(self.mowerStopSound, self.mowerStopSoundPitchOffset);
				playSample(self.mowerStopSound, 1, self.mowerStopSoundVolume, 0);					
				self.mowerLoopEndTime = self.time + self.mowerStopDuration;
			end;				
		end;
	end;	
	


end;

function PoettingerAlpha:onAttach(attacherVehicle)
	self.mowerIsAttached = true;
	self.attacherVehicle = attacherVehicle;
	self.attacherVehicleCopy = attacherVehicle;
	
	--local rx, ry, rz = getRotation(self.alpMot.halter.node);
	--local newVal = Utils.getMovedLimitedValues({rx, ry, rz}, self.alpMot.halter.rotMax, self.alpMot.halter.rotMin, 3, 4000, dt, self.alpMot.setTransport);
	setRotation(self.alpMot.halter.node, self.alpMot.halter.rotMax[1], self.alpMot.halter.rotMax[2], self.alpMot.halter.rotMax[3]);	
	
	if self.alpMot.isTransport then
		local x,y,z = getTranslation(self.componentJoints[1].jointNode);
	--{0.34199}, {0.50691}
		local newY = {0.34199}; --{-0.12023};
		setTranslation(self.componentJoints[1].jointNode, x, newY[1], z);
		setJointFrame(self.componentJoints[1].jointIndex, 0, self.componentJoints[1].jointNode);	
	end;
	
	setCenterOfMass( self.components[2].node, 0, 0, -1 );
end;

function PoettingerAlpha:onDetach()
	self.attacherVehicleCopy.needsLowering = true;
	self.attacherVehicle = nil;
	self.parentAttVehJoint = nil;
	setRotation(self.alpMot.halter.node, self.alpMot.halter.rotMin[1], self.alpMot.halter.rotMin[2], self.alpMot.halter.rotMin[3]);	
	
	local x,y,z = getTranslation(self.componentJoints[1].jointNode);
	local newY = {0.50691}; --{-0.28515}; --{-0.10664}
	setTranslation(self.componentJoints[1].jointNode, x, newY[1], z);
	setJointFrame(self.componentJoints[1].jointIndex, 0, self.componentJoints[1].jointNode);	

	setCenterOfMass( self.components[2].node, 0, 0, 0 );	
end;

function PoettingerAlpha:draw()
	g_currentMission:addHelpButtonText(string.format(g_i18n:getText("POETX_TurnOnOff_disp"), tostring(self.alpMot.setTurnedOn)), InputBinding.POETX_TURNONOFF );
	g_currentMission:addHelpButtonText(string.format(g_i18n:getText("POETX_Transport_disp"), tostring(self.alpMot.setTransport)), InputBinding.POETX_TRANSPORT);
end;


function PoettingerAlpha:setLiftUp(value, noEventSend)
	SetLiftUpEvent.sendEvent(self, value, noEventSend);
	self.alpMot.setLiftUp = value;

end;

function PoettingerAlpha:setTurnedOn(value, noEventSend)
	SetTurnedOnEvent.sendEvent(self, value, noEventSend);
	self.alpMot.setTurnedOn = value;
end;
 

function PoettingerAlpha:setTransport(value, noEventSend)
	SetTransportEvent.sendEvent(self, value, noEventSend);
	self.alpMot.setTransport = value;
end;
 
function PoettingerAlpha:setPSstate(value, noEventSend)
	SetPSstateEvent.sendEvent(self, value, noEventSend);
	self.psState = value;
end;


function PoettingerAlpha:setWorkingTransportState( _isWorking, _isTransport, _noEventSend)
	SetWorkingTransportStateEvent.sendEvent(self, _isWorking, _isTransport, _noEventSend)
	self.alpMot.isWorking = _isWorking;
	self.alpMot.isTransport = _isTransport;
	-- fix
	if self.isClient then
		if _isWorking then
			setRotation(self.alpMot.left.node, self.alpMot.left.rotMin[1], self.alpMot.left.rotMin[2], self.alpMot.left.rotMin[3]);
			setRotation(self.alpMot.right.node, self.alpMot.left.rotMin[1], self.alpMot.left.rotMin[2], -self.alpMot.left.rotMin[3]);	
			for i,plane in pairs(self.alpMot.dynPlanes) do
				if plane.scale then
					setScale(plane.node, 1, 1.0, 1);			
				end;
			end;
		elseif _isTransport then
			setRotation(self.alpMot.left.node, self.alpMot.left.rotMax[1], self.alpMot.left.rotMax[2], self.alpMot.left.rotMax[3]);
			setRotation(self.alpMot.right.node, self.alpMot.left.rotMax[1], self.alpMot.left.rotMax[2], -self.alpMot.left.rotMax[3]);	
			for i,plane in pairs(self.alpMot.dynPlanes) do
				if plane.scale then
					setScale(plane.node, 1, 0.5, 1);			
				end;
			end;
		end;
	end;
end;

function PoettingerAlpha:setLiftUpDownState( _isLiftUp, _isLiftDown, _noEventSend)
	SetLiftUpDownStateEvent.sendEvent(self, _isLiftUp, _isLiftDown, _noEventSend)
	self.alpMot.isLiftUp = _isLiftUp;
	self.alpMot.isLiftDown = _isLiftDown;
end;

 